/**
* Copyright (c) 2014 - 2018, Nordic Semiconductor ASA
* 
* All rights reserved.
* 
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
* 
* 1. Redistributions of source code must retain the above copyright notice, this
*    list of conditions and the following disclaimer.
* 
* 2. Redistributions in binary form, except as embedded into a Nordic
*    Semiconductor ASA integrated circuit in a product or a software update for
*    such product, must reproduce the above copyright notice, this list of
*    conditions and the following disclaimer in the documentation and/or other
*    materials provided with the distribution.
* 
* 3. Neither the name of Nordic Semiconductor ASA nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
* 
* 4. This software, with or without modification, must only be used with a
*    Nordic Semiconductor ASA integrated circuit.
* 
* 5. Any software provided in binary form under this license must not be reverse
*    engineered, decompiled, modified and/or disassembled.
* 
* THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
* OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
*/

#include <stdint.h>
#include <string.h>
#include "our_service.h"
#include "ble_srv_common.h"
#include "app_error.h"
#include "SEGGER_RTT.h"
#include "archive.h"
#include "main.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

uint8_t pass;
uint8_t real_time[] = {25, 3, 45};
uint8_t disc_client;
//uint8_t hadle_client;
bool action_evt = true;

uint16_t reg_cccd_handler;
void get_cccd_reg_serv(ble_reg_serv_t * p_our_service)
{
  reg_cccd_handler = p_our_service->char_handles.cccd_handle;
  NRF_LOG_INFO("REG CCCD");
}

//ble_gatts_evt_t test_gatts;
static void on_write(ble_evt_t const * p_ble_evt) 
{
  uint8_t notifying_enabled;
  NRF_LOG_INFO("REG ON WRITE");
  
  uint16_t p_evt_write_handle = p_ble_evt->evt.gatts_evt.params.write.handle;
  
  if ((p_evt_write_handle == reg_cccd_handler) && (p_ble_evt->evt.gatts_evt.params.write.len == 2)) 
  {
    NRF_LOG_INFO("CCCD Values %x, %x and Len %d", p_evt_write_handle, reg_cccd_handler, p_ble_evt->evt.gatts_evt.params.write.len);
    // CCCD written. Start notifications
    notifying_enabled = ble_srv_is_notification_enabled(p_ble_evt->evt.gatts_evt.params.write.data);
      
    if (notifying_enabled) 
    {
//      test_gatts = p_ble_evt->evt.gatts_evt;
//      start_send_config_flag = true;
      NRF_LOG_INFO("NOTIF ENABLE (REG) %d", notifying_enabled);
    }
    else 
    {
      NRF_LOG_INFO("NOTIF IS NOT ENABLE %d", notifying_enabled);
    }
  }
  else {NRF_LOG_INFO("CCCD Values (N_Equal) %x, %x and Len %d", p_ble_evt->evt.gatts_evt.params.write.handle, reg_cccd_handler, p_ble_evt->evt.gatts_evt.params.write.len);}
}

/**@brief Function for initiating our new service.
*
* @param[in]   p_reg_service        Our Service structure.
*
*/
void ble_registration_service_on_ble_evt(ble_evt_t const * p_ble_evt, void * p_context)
{
  ble_reg_serv_t * p_reg_service =(ble_reg_serv_t *) p_context;  
  
  switch (p_ble_evt->header.evt_id)
  {
    case BLE_GAP_EVT_CONNECTED:
    p_reg_service->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
    break;
    case BLE_GAP_EVT_DISCONNECTED:
    p_reg_service->conn_handle = BLE_CONN_HANDLE_INVALID;
    break;
    case BLE_GATTS_EVT_WRITE:
    on_write(p_ble_evt);
    if (p_ble_evt->evt.gatts_evt.params.write.uuid.uuid == 0xDAFF)
    {
      //      if ((p_ble_evt->evt.gatts_evt.params.write.data[0]&0x02) == 0)
      //      {
      //        disc_client = p_ble_evt->evt.gatts_evt.params.write.data[0];
      //        if (disc_client == 1)
      //        {NRF_LOG_INFO("Flag Disc Client = %d", time_conn[0]); action_evt = true; set_time_duration(1);}
      //      }
    }
    break;
    default:
    // No implementation needed.
    break;
  }
}

void reg_char_add(ble_reg_serv_t * p_reg_service)
{
  uint32_t   err_code;
  
  ble_uuid_t        char_uuid;
  ble_uuid128_t     base_uuid = BLE_UUID_REGISTR_BASE_UUID;
  char_uuid.uuid = BLE_UUID_REGISTR_WRITE_UUID;
  err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
  APP_ERROR_CHECK(err_code);
  
  //Add read/write properties to our characteristic value
  ble_gatts_char_md_t char_md;
  memset(&char_md, 0, sizeof(char_md));
  char_md.char_props.read     = 1;
  char_md.char_props.write    = 1;
  
  //Configuring CCCD metadata
  ble_gatts_attr_md_t cccd_md;
  memset(&cccd_md, 0, sizeof(cccd_md));
  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
  cccd_md.vloc                = BLE_GATTS_VLOC_STACK;
  char_md.p_cccd_md           = &cccd_md;
  char_md.char_props.notify   = 1;
  char_md.char_props.indicate = 0;
  
  //Configure the Attribute Metadata
  ble_gatts_attr_md_t attr_md;
  memset(&attr_md,0,sizeof(attr_md));
  attr_md.vloc = BLE_GATTS_VLOC_STACK;
  
  //Set read/write permissions to our characteristic
  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
  
  //Configure the Characteristic Value Attribute
  ble_gatts_attr_t attr_char_value;
  memset(&attr_char_value,0,sizeof(attr_char_value));
  attr_char_value.p_uuid      = &char_uuid;
  attr_char_value.p_attr_md   = &attr_md;
  attr_char_value.max_len     = 4;
  attr_char_value.init_len    = 4;
//  uint8_t value               = temp;
//  attr_char_value.p_value     = &value;
  
  //Add the new characteristic to the service
  err_code = sd_ble_gatts_characteristic_add(p_reg_service->service_handle, 
                                             &char_md,
                                             &attr_char_value,
                                             &p_reg_service->char_handles);
  
  //**********************
//  //New characteristic
//  //ble_uuid_t        char_uuid;
//  char_uuid.uuid = BLE_UUID_REGISTR_READ_UUID;
//  err_code = sd_ble_uuid_vs_add(&base_uuid, &char_uuid.type);
//  APP_ERROR_CHECK(err_code);
//  
//  //Add read/write properties to our characteristic value
//  //ble_gatts_char_md_t char_md;
//  memset(&char_md, 0, sizeof(char_md));
//  char_md.char_props.read     = 0;
//  char_md.char_props.write    = 1;
//  
//  //Configuring CCCD metadata
//  //ble_gatts_attr_md_t cccd_md;
//  memset(&cccd_md, 0, sizeof(cccd_md));
//  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
//  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
//  cccd_md.vloc                = BLE_GATTS_VLOC_STACK;
//  char_md.p_cccd_md           = &cccd_md;
//  char_md.char_props.notify   = 0;
//  char_md.char_props.indicate = 0;
//  
//  //Configure the Attribute Metadata
//  //ble_gatts_attr_md_t attr_md;
//  memset(&attr_md,0,sizeof(attr_md));
//  attr_md.vloc = BLE_GATTS_VLOC_STACK;
//  
//  //Set read/write permissions to our characteristic
//  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
//  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
//  
//  //Configure the Characteristic Value Attribute
//  //ble_gatts_attr_t attr_char_value;
//  memset(&attr_char_value,0,sizeof(attr_char_value));
//  attr_char_value.p_uuid      = &char_uuid;
//  attr_char_value.p_attr_md   = &attr_md;
//  attr_char_value.max_len     = 5;
//  attr_char_value.init_len    = 5;
////  uint8_t value_2[8]          = {0x21, 0x31, 0x54, 0x64, 0x87, 0x97};
////  attr_char_value.p_value     = value_2;
//  
//  //Add the new characteristic to the service
//  err_code = sd_ble_gatts_characteristic_add(p_reg_service->service_handle, 
//                                             &char_md,
//                                             &attr_char_value,
//                                             &p_reg_service->char_handles);
  
}

void reg_service_init(ble_reg_serv_t * p_reg_service)
{
  
  // STEP 3: Declare 16 bit service and 128 bit base UUIDs and add them to BLE stack table     
  // STEP 4: Add our service
  
  uint32_t   err_code;
  
  ble_uuid_t        service_uuid;
  ble_uuid128_t     base_uuid = BLE_UUID_REGISTR_BASE_UUID;
  service_uuid.uuid = BLE_UUID_REGISTR_SERVICE;
  err_code = sd_ble_uuid_vs_add(&base_uuid, &service_uuid.type);
  APP_ERROR_CHECK(err_code);
  
  p_reg_service->conn_handle = BLE_CONN_HANDLE_INVALID;
  
  err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY,
                                      &service_uuid,
                                      &p_reg_service->service_handle);
  APP_ERROR_CHECK(err_code);
  
  reg_char_add(p_reg_service);
}

//Update characteristic value
void send_synchro_param(ble_reg_serv_t *p_reg_service, uint8_t *parametrs)
{
  uint16_t               len = 4;
  ble_gatts_hvx_params_t hvx_params;
  memset(&hvx_params, 0, sizeof(hvx_params));
  
  hvx_params.handle = p_reg_service->char_handles.value_handle;
  hvx_params.offset = 0;
  hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
  hvx_params.p_len  = &len;
  hvx_params.p_data = parametrs;
  uint32_t err_code = sd_ble_gatts_hvx(p_reg_service->conn_handle, &hvx_params);
  APP_ERROR_CHECK(err_code);
}

